@@ -18,6 +18,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 
@@ -75,6 +79,12 @@ static int msopenh264_dec_reset_first_image(MSFilter *f, void *arg) {
 	return 0;
 }
 
+static int msopenh264_dec_enable_avpf(MSFilter *f, void *arg) {
+	MSOpenH264Decoder *d = static_cast<MSOpenH264Decoder *>(f->data);
+	d->enableAVPF(*((bool_t *)arg) ? true : false);
+	return 0;
+}
+
 static int msopenh264_dec_get_size(MSFilter *f, void *arg) {
 	MSOpenH264Decoder *d = static_cast<MSOpenH264Decoder *>(f->data);
 	MSVideoSize *size = static_cast<MSVideoSize *>(arg);
@@ -98,6 +108,7 @@ static int msopenh264_dec_get_out_fmt(MSFilter *f, void *arg){
 static MSFilterMethod msopenh264_dec_methods[] = {
 	{ MS_FILTER_ADD_FMTP,                              msopenh264_dec_add_fmtp          },
 	{ MS_VIDEO_DECODER_RESET_FIRST_IMAGE_NOTIFICATION, msopenh264_dec_reset_first_image },
+	{ MS_VIDEO_DECODER_ENABLE_AVPF,                    msopenh264_dec_enable_avpf       },
 	{ MS_FILTER_GET_VIDEO_SIZE,                        msopenh264_dec_get_size          },
 	{ MS_FILTER_GET_FPS,                               msopenh264_dec_get_fps           },
 	{ MS_FILTER_GET_OUTPUT_FMT,                        msopenh264_dec_get_out_fmt       },
@@ -108,54 +119,23 @@ static MSFilterMethod msopenh264_dec_methods[] = {
  * Definition of the decoder                                                  *
  *****************************************************************************/
 
-#define MSOPENH264_DEC_NAME        "MSOpenH264Dec"
-#define MSOPENH264_DEC_DESCRIPTION "OpenH264 Video Codec provided by Cisco Systems, Inc."
-#define MSOPENH264_DEC_CATEGORY    MS_FILTER_DECODER
-#define MSOPENH264_DEC_ENC_FMT     "H264"
-#define MSOPENH264_DEC_NINPUTS     1
-#define MSOPENH264_DEC_NOUTPUTS    1
-#define MSOPENH264_DEC_FLAGS       0
-
-#if 0
-
 MSFilterDesc msopenh264_dec_desc = {
-	.id = MS_FILTER_PLUGIN_ID,
-	.name = MSOPENH264_DEC_NAME,
-	.text = MSOPENH264_DEC_DESCRIPTION,
-	.category = MSOPENH264_DEC_CATEGORY,
-	.enc_fmt = MSOPENH264_DEC_ENC_FMT,
-	.ninputs = MSOPENH264_DEC_NINPUTS,
-	.noutputs = MSOPENH264_DEC_NOUTPUTS,
-	.init = msopenh264_dec_init,
-	.preprocess = msopenh264_dec_preprocess,
-	.process = msopenh264_dec_process,
-	.postprocess = NULL,
-	.uninit = msopenh264_dec_uninit,
-	.methods = msopenh264_dec_methods,
-	.flags = MSOPENH264_DEC_FLAGS
+	MS_FILTER_PLUGIN_ID, /* id */
+	"MSOpenH264Dec", /* name */
+	"OpenH264 Video Codec provided by Cisco Systems, Inc.", /* text */
+	MS_FILTER_DECODER, /* category */
+	"H264", /* enc_fmt */
+	1, /* ninputs */
+	1, /* noutputs */
+	msopenh264_dec_init, /* init */
+	msopenh264_dec_preprocess, /* preprocess */
+	msopenh264_dec_process, /* process */
+	NULL, /* postprocess */
+	msopenh264_dec_uninit, /* uninit */
+	msopenh264_dec_methods, /* methods */
+	0 /* flags */
 };
 
-#else
-
-MSFilterDesc msopenh264_dec_desc = {
-	MS_FILTER_PLUGIN_ID,
-	MSOPENH264_DEC_NAME,
-	MSOPENH264_DEC_DESCRIPTION,
-	MSOPENH264_DEC_CATEGORY,
-	MSOPENH264_DEC_ENC_FMT,
-	MSOPENH264_DEC_NINPUTS,
-	MSOPENH264_DEC_NOUTPUTS,
-	msopenh264_dec_init,
-	msopenh264_dec_preprocess,
-	msopenh264_dec_process,
-	NULL,
-	msopenh264_dec_uninit,
-	msopenh264_dec_methods,
-	MSOPENH264_DEC_FLAGS
-};
-
-#endif
-
 
 /******************************************************************************
  * Implementation of the encoder                                              *
@@ -246,6 +226,25 @@ static int msopenh264_enc_req_vfu(MSFilter *f, void *arg) {
 	return 0;
 }
 
+static int msopenh264_enc_notify_pli(MSFilter *f, void *arg) {
+	MSOpenH264Encoder *e = static_cast<MSOpenH264Encoder *>(f->data);
+	e->notifyPLI();
+	return 0;
+}
+
+static int msopenh264_enc_notify_fir(MSFilter *f, void *arg) {
+	MSOpenH264Encoder *e = static_cast<MSOpenH264Encoder *>(f->data);
+	uint8_t seqnr = *((uint8_t *)arg);
+	e->notifyFIR(seqnr);
+	return 0;
+}
+
+static int msopenh264_enc_enable_avpf(MSFilter *f, void *arg) {
+	MSOpenH264Encoder *e = static_cast<MSOpenH264Encoder *>(f->data);
+	e->enableAVPF(*((bool_t *)arg) ? true : false);
+	return 0;
+}
+
 static int msopenh264_enc_get_configuration_list(MSFilter *f, void *arg) {
 	MSOpenH264Encoder *e = static_cast<MSOpenH264Encoder *>(f->data);
 	const MSVideoConfiguration **vconf_list = static_cast<const MSVideoConfiguration **>(arg);
@@ -253,6 +252,13 @@ static int msopenh264_enc_get_configuration_list(MSFilter *f, void *arg) {
 	return 0;
 }
 
+static int msopenh264_enc_set_configuration_list(MSFilter *f, void *arg) {
+	MSOpenH264Encoder *e = static_cast<MSOpenH264Encoder *>(f->data);
+	const MSVideoConfiguration **vconf_list = static_cast<const MSVideoConfiguration **>(arg);
+	e->setConfigurationList(*vconf_list);
+	return 0;
+}
+
 static int msopenh264_enc_set_configuration(MSFilter *f, void *arg) {
 	MSOpenH264Encoder *e = static_cast<MSOpenH264Encoder *>(f->data);
 	MSVideoConfiguration *vconf = static_cast<MSVideoConfiguration *>(arg);
@@ -269,10 +275,12 @@ static MSFilterMethod msopenh264_enc_methods[] = {
 	{ MS_FILTER_GET_VIDEO_SIZE,                msopenh264_enc_get_vsize              },
 	{ MS_FILTER_ADD_FMTP,                      msopenh264_enc_add_fmtp               },
 	{ MS_FILTER_REQ_VFU,                       msopenh264_enc_req_vfu                },
-#ifdef MS_VIDEO_ENCODER_REQ_VFU
 	{ MS_VIDEO_ENCODER_REQ_VFU,                msopenh264_enc_req_vfu                },
-#endif
+	{ MS_VIDEO_ENCODER_NOTIFY_PLI,             msopenh264_enc_notify_pli             },
+	{ MS_VIDEO_ENCODER_NOTIFY_FIR,             msopenh264_enc_notify_fir             },
+	{ MS_VIDEO_ENCODER_ENABLE_AVPF,            msopenh264_enc_enable_avpf            },
 	{ MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, msopenh264_enc_get_configuration_list },
+	{ MS_VIDEO_ENCODER_SET_CONFIGURATION_LIST, msopenh264_enc_set_configuration_list },
 	{ MS_VIDEO_ENCODER_SET_CONFIGURATION,      msopenh264_enc_set_configuration      },
 	{ 0,                                       NULL                                  }
 };
@@ -281,54 +289,23 @@ static MSFilterMethod msopenh264_enc_methods[] = {
  * Definition of the encoder                                                  *
  *****************************************************************************/
 
-#define MSOPENH264_ENC_NAME        "MSOpenH264Enc"
-#define MSOPENH264_ENC_DESCRIPTION "OpenH264 Video Codec provided by Cisco Systems, Inc."
-#define MSOPENH264_ENC_CATEGORY    MS_FILTER_ENCODER
-#define MSOPENH264_ENC_ENC_FMT     "H264"
-#define MSOPENH264_ENC_NINPUTS     1
-#define MSOPENH264_ENC_NOUTPUTS    1
-#define MSOPENH264_ENC_FLAGS       0
-
-#if 0
-
 MSFilterDesc msopenh264_enc_desc = {
-	.id = MS_FILTER_PLUGIN_ID,
-	.name = MSOPENH264_ENC_NAME,
-	.text = MSOPENH264_ENC_DESCRIPTION,
-	.category = MSOPENH264_ENC_CATEGORY,
-	.enc_fmt = MSOPENH264_ENC_ENC_FMT,
-	.ninputs = MSOPENH264_ENC_NINPUTS,
-	.noutputs = MSOPENH264_ENC_NOUTPUTS,
-	.init = msopenh264_enc_init,
-	.preprocess = msopenh264_enc_preprocess,
-	.process = msopenh264_enc_process,
-	.postprocess = msopenh264_enc_postprocess,
-	.uninit = msopenh264_enc_uninit,
-	.methods = msopenh264_enc_methods,
-	.flags = MSOPENH264_ENC_FLAGS
+	MS_FILTER_PLUGIN_ID, /* id */
+	"MSOpenH264Enc", /* name */
+	"OpenH264 Video Codec provided by Cisco Systems, Inc.", /* text */
+	MS_FILTER_ENCODER, /* category */
+	"H264", /* enc_fmt */
+	1, /* ninputs */
+	1, /* noutputs */
+	msopenh264_enc_init, /* init */
+	msopenh264_enc_preprocess, /* preprocess */
+	msopenh264_enc_process, /* process */
+	msopenh264_enc_postprocess, /* postprocess */
+	msopenh264_enc_uninit, /* uninit */
+	msopenh264_enc_methods, /* methods */
+	0 /* flags */
 };
 
-#else
-
-MSFilterDesc msopenh264_enc_desc = {
-	MS_FILTER_PLUGIN_ID,
-	MSOPENH264_ENC_NAME,
-	MSOPENH264_ENC_DESCRIPTION,
-	MSOPENH264_ENC_CATEGORY,
-	MSOPENH264_ENC_ENC_FMT,
-	MSOPENH264_ENC_NINPUTS,
-	MSOPENH264_ENC_NOUTPUTS,
-	msopenh264_enc_init,
-	msopenh264_enc_preprocess,
-	msopenh264_enc_process,
-	msopenh264_enc_postprocess,
-	msopenh264_enc_uninit,
-	msopenh264_enc_methods,
-	MSOPENH264_ENC_FLAGS
-};
-
-#endif
-
 
 #ifdef _MSC_VER
 #define MS_PLUGIN_DECLARE(type) extern "C" __declspec(dllexport) type
