@@ -31,37 +31,8 @@ static int debugLevel = 1;
 #endif
 
 
-VideoStarter::VideoStarter()
-	: mActive(true), mNextTime(0), mFrameCount(0)
-{}
+static const int MIN_KEY_FRAME_DIST = 4;
 
-VideoStarter::~VideoStarter()
-{}
-
-void VideoStarter::firstFrame(uint64_t curtime)
-{
-	mNextTime = curtime + 2000;
-}
-
-bool VideoStarter::needIFrame(uint64_t curtime)
-{
-	if (!mActive || (mNextTime == 0)) return false;
-	if (curtime >= mNextTime) {
-		mFrameCount++;
-		if (mFrameCount == 1) {
-			mNextTime += 2000;
-		} else {
-			mNextTime = 0;
-		}
-		return true;
-	}
-	return false;
-}
-
-void VideoStarter::deactivate()
-{
-	mActive = false;
-}
 
 #if defined(ANDROID) || (TARGET_OS_IPHONE == 1) || defined(__arm__)
 	#define MS_OPENH264_CONF(required_bitrate, bitrate_limit, resolution, fps_pc, cpus_pc, fps_mobile, cpus_mobile) \
@@ -96,14 +67,14 @@ static const MSVideoConfiguration openh264_conf_list[] = {
 
 
 MSOpenH264Encoder::MSOpenH264Encoder(MSFilter *f)
-	: mFilter(f), mPacker(0), mPacketisationMode(0), mVConfList(openh264_conf_list), mFrameCount(0), mInitialized(false), mPacketisationModeSet(false)
+	: mFilter(f), mPacker(0), mPacketisationMode(0), mVConfList(openh264_conf_list), mFrameCount(0), mLastIDRFrameCount(0),
+		mInitialized(false), mPacketisationModeSet(false), mAVPFEnabled(false), mLastFIRSeqNr(-1)
 {
-	mVConf = ms_video_find_best_configuration_for_bitrate(mVConfList, 384000, ms_factory_get_cpu_count(mFilter->factory));
-
 	long ret = WelsCreateSVCEncoder(&mEncoder);
 	if (ret != 0) {
 		ms_error("OpenH264 encoder: Failed to create encoder: %li", ret);
 	}
+	setConfigurationList(NULL);
 }
 
 MSOpenH264Encoder::~MSOpenH264Encoder()
@@ -121,6 +92,7 @@ void MSOpenH264Encoder::initialize()
 		rfc3984_set_mode(mPacker, mPacketisationMode);
 	else rfc3984_set_mode(mPacker, 1); // in absence of explicit directive from the other end, allow mode 1 because it allows to send big slices which is necessary for large images
 	rfc3984_enable_stap_a(mPacker, FALSE);
+
 	if (mEncoder != 0) {
 		SEncParamExt params;
 		int ret = mEncoder->GetDefaultParams(&params);
@@ -149,6 +121,7 @@ void MSOpenH264Encoder::initialize()
 			params.bEnableSceneChangeDetect = false;
 			params.bEnableLongTermReference  = false;
 			params.iSpatialLayerNum=1;
+			params.eSpsPpsIdStrategy = CONSTANT_ID;
 			
 			params.sSpatialLayers[0].iVideoWidth = mVConf.vsize.width;
 			params.sSpatialLayers[0].iVideoHeight = mVConf.vsize.height;
@@ -170,6 +143,9 @@ void MSOpenH264Encoder::initialize()
 			}
 		}
 	}
+	if (!mAVPFEnabled && (mFrameCount == 0)) {
+		ms_video_starter_init(&mVideoStarter);
+	}
 }
 
 void MSOpenH264Encoder::feed()
@@ -198,18 +174,20 @@ void MSOpenH264Encoder::feed()
 				srcPic.pData[i] = pic.planes[i];
 			}
 			srcPic.uiTimeStamp = ts;
-			// Send I frame 2 seconds and 4 seconds after the beginning
-			if (mVideoStarter.needIFrame(mFilter->ticker->time)) {
-					generateKeyframe();
+			if (!mAVPFEnabled && ms_video_starter_need_i_frame(&mVideoStarter, mFilter->ticker->time)) {
+				generateKeyframe();
 			}
 
 			int ret = mEncoder->EncodeFrame(&srcPic, &sFbi);
 			if (ret == cmResultSuccess) {
 				if ((sFbi.eFrameType != videoFrameTypeSkip) && (sFbi.eFrameType != videoFrameTypeInvalid)) {
-					if (mFrameCount == 0) {
-						mVideoStarter.firstFrame(mFilter->ticker->time);
+					if (sFbi.eFrameType == videoFrameTypeIDR) {
+						mLastIDRFrameCount = mFrameCount;
 					}
 					mFrameCount++;
+					if (!mAVPFEnabled && (mFrameCount == 1)) {
+						ms_video_starter_first_frame(&mVideoStarter, mFilter->ticker->time);
+					}
 					fillNalusQueue(sFbi, &nalus);
 					rfc3984_pack(mPacker, &nalus, mFilter->outputs[0], sFbi.uiTimeStamp);
 				}
@@ -270,6 +248,17 @@ void MSOpenH264Encoder::addFmtp(const char *fmtp)
 	}
 }
 
+void MSOpenH264Encoder::setConfigurationList(const MSVideoConfiguration *confList) {
+	MSVideoSize vsize;
+	if (confList == NULL) {
+		mVConfList = openh264_conf_list;
+	} else {
+		mVConfList = confList;
+	}
+	MS_VIDEO_SIZE_ASSIGN(vsize, CIF);
+	mVConf = ms_video_find_best_configuration_for_size(mVConfList, vsize, ms_factory_get_cpu_count(mFilter->factory));
+}
+
 void MSOpenH264Encoder::setConfiguration(MSVideoConfiguration conf)
 {
 	mVConf = conf;
@@ -289,10 +278,27 @@ void MSOpenH264Encoder::setConfiguration(MSVideoConfiguration conf)
 void MSOpenH264Encoder::requestVFU()
 {
 	// If we receive a VFU request, stop the video starter
-	mVideoStarter.deactivate();
+	ms_video_starter_deactivate(&mVideoStarter);
 	generateKeyframe();
 }
 
+void MSOpenH264Encoder::notifyPLI()
+{
+	ms_message("OpenH264: PLI requested");
+	if (shouldGenerateKeyframe(MIN_KEY_FRAME_DIST)) {
+		ms_message("OpenH264: PLI accepted");
+		generateKeyframe();
+	}
+}
+
+void MSOpenH264Encoder::notifyFIR(uint8_t seqnr)
+{
+	if (seqnr != mLastFIRSeqNr) {
+		mLastFIRSeqNr = seqnr;
+		generateKeyframe();
+	}
+}
+
 void MSOpenH264Encoder::generateKeyframe()
 {
 	if (isInitialized()) {
@@ -308,6 +314,12 @@ void MSOpenH264Encoder::generateKeyframe()
 	}
 }
 
+bool MSOpenH264Encoder::shouldGenerateKeyframe(int frameDist)
+{
+	if (mFrameCount > mLastIDRFrameCount + frameDist) return true;
+	return false;
+}
+
 void MSOpenH264Encoder::fillNalusQueue(SFrameBSInfo &sFbi, MSQueue *nalus)
 {
 	for (int i = 0; i < sFbi.iLayerNum; i++) {
